(in-package #:p12f675)

(with-pic

	(config _CP_OFF _CPD_OFF _BODEN_OFF _MCLRE_OFF _WDT_OFF 
			_PWRTE_ON _INTRC_OSC_NOCLKOUT)

	;; Uninitialized variable space starting at address #x20
	(cblock #20
		(w_temp 1)
		(status_temp 1)
		(dvar 1)
		(dvar2 1)
		(save_w))

	(.org		#x000)			; processor reset vector
	(.goto		@Main)			; go to beginning of program
	
	(.org		#x004)			; interrupt vector location
	(.movwf		$w_temp)		; save off current W register contents
	(.movf		STATUS W)		; move status register into W register
	(.movwf		$status_temp)	; save off contents of STATUS register

	;; isr code can go here or be located as a call subroutine elsewhere

	(.movf		$status_temp W)	; retrieve copy of STATUS register
	(.movwf		STATUS)			; restore pre-isr STATUS register contents
	(.swapf		$w_temp F)
	(.swapf		$w_temp W)		; restore pre-isr W register contents
	(.retfie)					; return from interrupt


;; these first 4 instructions are required if the internal oscillator is used
@Main
	(.call		#x3FF)			; retrieve factory calibration value
	(.bsf		STATUS RP0)		; set file register bank to 1 
	(.movwf		OSCCAL)			; update register with factory cal value 
	(.bcf		STATUS RP0)		; set file register bank to 0

;; remaining code goes here

	(.call		@InitGPIO)

@Loop
	(.call		@TurnOnLed)
	(.call		@Delay)
	(.call		@TurnOffLed)
	(.call		@Delay)
	(.goto		@Loop)		; infinite loop

@Delay
	(.movlw		#xA0)		; set outer delay loop
	(.movwf		$dvar2)
@Delay0
	(.movlw		#xFF)
	(.movwf		$dvar)		; set inner delay loop
@Delay1
	(.decfsz	$dvar)
	(.goto		@Delay1)

	(.decfsz	$dvar2)
	(.goto		@Delay0)
	
	(.return)

;; Subroutine to init GPIO to all outputs, except for GP<3> which shall only
;; be an input pin.
@InitGPIO
	(.movwf		$save_w)
	(.bcf		STATUS RP0)		; Bank 0
	(.clrf		GPIO)			; Init GPIO
	(.movlw		#x07)			; Set GP<2:0> to
	(.movwf		CMCON)			; digital I/O
	(.bsf		STATUS RP0)		; Bank 1
	(.clrf		ANSEL)			; Digital I/O
	(.movlw		#x80)			; Set GP<3> as input, GP<5:4,2:0> as output
	(.movwf		TRISIO)
	(.bcf		STATUS RP0)		; Bank 0
	(.movfw		$save_w)		
	(.return)

@TurnOnLed
	(.movwf		$save_w)
	(.movlw		#x02)
	(.movwf		GPIO)			; Turn on GP<1>
	(.movfw		$save_w)
	(.return)

@TurnOffLed
	(.movwf		$save_w)
	(.movlw		#x00)
	(.movwf		GPIO)			; Turn off GP<1>
	(.movfw		$save_w)
	(.return)

;; initialize eeprom locations

	(.org	#x2100)
	(.de	#x00  #x01  #x02  #x03))
