(in-package #:pic-base)

(defmacro bits (max &optional to (min max))
  (declare (ignore to))
  `(byte (- ,max ,min -1) ,min))

;; This is a single compilable unit. It is an instruction or directive.
(defstruct unit
  ;; can be :instruction, :directive, :label
  kind
  ;; the name of the thing in question
  name
  ;; A thunk representing the operands in lexical context
  operand-func
  ;; A function which will produce the binary output of this
  ;; (potential) opcode when supplied with literal arguments.
  assembly-func
  )

;; Each compilation-unit reresents a stream of elements and the
;; environment under which it will compile. If there is only one
;; compilation-unit, then it'll be for absolute compilation. The
;; assembler/linker phase takes care of collecting such things as an
;; overlay sections and whatnot out of all of the compilation-units.
(defstruct compilation-unit
  ;; Is it :relocatable or :absolute?
  style
  ;; A list of elements...
  units
  ;; ...which use this environment for compilation.
  environment
  )

;; a hash-table of all compilation-units and their environments. Each
;; key in the hash table represents one (possibly relocatable)
;; compilation-unit.
(defvar *compilation-units*)

;; In phase 1 of the assembler, all label references which aren't in
;; operand thunks get changed into label units and inserted into the
;; current compilation unit.

;; NOTE! This is the working read-macro. See the broken one below.

#|
(eval-when (:compile-toplevel :load-toplevel :execute)
  (set-macro-character
   #\@
   #'(lambda (stream char)
       (declare (ignore char))
       (let ((obj (read stream t nil t)))
         (assert (symbolp obj))
         `(restart-case
              (handler-bind ((warning
                              #'(lambda (c)
                                  (declare (ignore c))
                                  (muffle-warning))))
                (locally #+sbcl(declare (sb-ext:muffle-conditions warning))
                         (format t "About to evaluate unbound variable ~A~%"
                                 ',obj)
                         (finish-output)
                         ;; XXX Apparently, SBCL has some kind of an ;
                         ;; optimization bug where it will notice that ;
                         ;; ,obj isn't ever actually used in the ;
                         ;; compilation-units expression and therefore
                         ;; it won't even evaluate it if I just use a
                         ;; bare ,obj as the return of this locally
                         ;; form! This causes problems because then I
                         ;; don't get the unbound-variable signal and
                         ;; side effect the label generated by the
                         ;; signal into the compilation unit. So, this
                         ;; is a slow, but temporary, workaround until
                         ;; I can figure out what to do about it.
                         (let ((g ,obj))
                           (format nil "About to eval undefined label ~A~%"
                                   g)
                           g)))
            (:use-value (value)
              (format t "Label fix up with ~A~%" value)
              value))))))

|#

;;#|
;; NOTE! This is the broken read-macro.
(eval-when (:compile-toplevel :load-toplevel :execute)
  (set-macro-character
   #\@
   #'(lambda (stream char)
       (declare (ignore char))
       (let ((obj (read stream t nil t)))
         (assert (symbolp obj))
         `(restart-case
              (handler-bind ((warning
                              #'(lambda (c)
                                  (declare (ignore c))
                                  (muffle-warning))))
                (locally #+sbcl(declare (sb-ext:muffle-conditions warning))
                         (format t "About to evaluate unbound variable ~A~%"
                                 ',obj)
                         (finish-output)
                         ,obj))
            (:use-value (value)
              (format t "Label fix up with ~A~%" value)
              value))))))
;;|#
